/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Spatial SDK, Version 6.0                   */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Odometer Configuration */
public class ANPacket192
{
	public static final int LENGTH = 8;

	public Boolean permanent;
	public Boolean automaticPulseMeasurementActive;
	public Boolean stationaryPositionLock;
	public float pulseLength;

	public ANPacket192()
	{
	}

	public ANPacket192(ANPacket packet)
	{
		permanent = packet.data[0] != 0;
		automaticPulseMeasurementActive = (packet.data[1]&0x01) != 0;
		stationaryPositionLock = (packet.data[1]&0x02) != 0;
		pulseLength = TypeConversion.bytesToFloat(packet.data, 4);
	}

	public ANPacket toANPacket() throws IndexOutOfBoundsException
	{	
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_ODOMETER_CONFIGURATION);
		packet.data[0] = (byte) (permanent ? 1 : 0);
		packet.data[1] = (byte) ((automaticPulseMeasurementActive ? 0x01 : 0x00) | (stationaryPositionLock ? 0x02 : 0x00));
		for(int i=2; i < 4; i++)
		{
			packet.data[i] = 0;
		}
		TypeConversion.floatToBytes(packet.data, 4, pulseLength);
		return packet;
	}
}
