/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Spatial SDK, Version 6.0                   */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Raw GNSS */
public class ANPacket29
{
	public static final int LENGTH = 74;

	public static final int GNSS_NO_FIX = 0;
	public static final int GNSS_2D_FIX = 1;
	public static final int GNSS_3D_FIX = 2;
	public static final int GNSS_SBAS_FIX = 3;
	public static final int GNSS_DIFFERENTIAL_FIX = 4;
	public static final int GNSS_OMNISTAR_FIX = 5;
	public static final int GNSS_RTK_FLOAT_FIX = 6;
	public static final int GNSS_RTK_FIXED_FIX = 7;

	public long unixTimeStamp;
	public long microseconds;
	public double latitude;
	public double longitude;
	public double height;
	public float velocityNorth;
	public float velocityEast;
	public float velocityDown;
	public float latitudeStandardDeviation;
	public float longitudeStandardDeviation;
	public float heightStandardDeviation;
	public float tilt;
	public float heading;
	public float tiltStandardDeviation;
	public float headingStandardDeviation;

	public int fixType;
	public Boolean dopplerVelocityValid;
	public Boolean timeValid;
	public Boolean externalGNSS;
	public Boolean tiltValid;
	public Boolean headingValid;

	public Boolean floatingHeading;
	public Boolean antenna1Disconnected;
	public Boolean antenna2Disconnected;
	public Boolean antenna1Short;
	public Boolean antenna2Short;
	public Boolean gnss1Failure;
	public Boolean gnss2Failure;

	public ANPacket29()
	{
	}

	public ANPacket29(ANPacket packet)
	{
		unixTimeStamp = TypeConversion.bytesToUInt32(packet.data, 0);
		microseconds = TypeConversion.bytesToUInt32(packet.data, 4);
		latitude = TypeConversion.bytesToDouble(packet.data, 8);
		longitude = TypeConversion.bytesToDouble(packet.data, 16);
		height = TypeConversion.bytesToDouble(packet.data, 24);
		velocityNorth = TypeConversion.bytesToFloat(packet.data, 32);
		velocityEast = TypeConversion.bytesToFloat(packet.data, 36);
		velocityDown = TypeConversion.bytesToFloat(packet.data, 40);
		latitudeStandardDeviation = TypeConversion.bytesToFloat(packet.data, 44);
		longitudeStandardDeviation = TypeConversion.bytesToFloat(packet.data, 48);
		heightStandardDeviation = TypeConversion.bytesToFloat(packet.data, 52);
		tilt = TypeConversion.bytesToFloat(packet.data, 56);
		heading = TypeConversion.bytesToFloat(packet.data, 60);
		tiltStandardDeviation = TypeConversion.bytesToFloat(packet.data, 64);
		headingStandardDeviation = TypeConversion.bytesToFloat(packet.data, 68);

		fixType = packet.data[72] & 0x07;
		dopplerVelocityValid = (packet.data[72] & 0x08) != 0;
		timeValid = (packet.data[72] & 0x10) != 0;
		externalGNSS = (packet.data[72] & 0x20) != 0;
		tiltValid = (packet.data[72] & 0x40) != 0;
		headingValid = (packet.data[72] & 0x80) != 0;

		floatingHeading = (packet.data[73] & 0x01) != 0;
		antenna1Disconnected = (packet.data[73] & (1<<2)) != 0;
		antenna2Disconnected = (packet.data[73] & (1<<3)) != 0;
		antenna1Short = (packet.data[73] & (1<<4)) != 0;
		antenna2Short = (packet.data[73] & (1<<5)) != 0;
		gnss1Failure = (packet.data[73] & (1<<6)) != 0;
		gnss2Failure = (packet.data[73] & (1<<7)) != 0;
	}
	
	public ANPacket toANPacket()
	{
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_RAW_GNSS);
		TypeConversion.uint32ToBytes(packet.data, 0, unixTimeStamp);
		TypeConversion.uint32ToBytes(packet.data, 4, microseconds);
		TypeConversion.doubleToBytes(packet.data, 8, latitude);
		TypeConversion.doubleToBytes(packet.data, 16, longitude);
		TypeConversion.doubleToBytes(packet.data, 24, height);
		TypeConversion.floatToBytes(packet.data, 32, velocityNorth);
		TypeConversion.floatToBytes(packet.data, 36, velocityEast);
		TypeConversion.floatToBytes(packet.data, 40, velocityDown);
		TypeConversion.floatToBytes(packet.data, 44, latitudeStandardDeviation);
		TypeConversion.floatToBytes(packet.data, 48, longitudeStandardDeviation);
		TypeConversion.floatToBytes(packet.data, 52, heightStandardDeviation);
		TypeConversion.floatToBytes(packet.data, 56, tilt);
		TypeConversion.floatToBytes(packet.data, 60, heading);
		TypeConversion.floatToBytes(packet.data, 64, tiltStandardDeviation);
		TypeConversion.floatToBytes(packet.data, 68, headingStandardDeviation);

		packet.data[72] = (byte) (fixType & 0x07);
		packet.data[72] |= (byte) (dopplerVelocityValid ? 0x08 : 0);
		packet.data[72] |= (byte) (timeValid ? 0x10 : 0);
		packet.data[72] |= (byte) (externalGNSS ? 0x20 : 0);
		packet.data[72] |= (byte) (tiltValid ? 0x40 : 0);
		packet.data[72] |= (byte) (headingValid ? 0x80 : 0);

		packet.data[73] = (byte) (floatingHeading ? 0x01 : 0);
		packet.data[73] |= (byte) (antenna1Disconnected ? 0x04 : 0);
		packet.data[73] |= (byte) (antenna2Disconnected ? 0x08 : 0);
		packet.data[73] |= (byte) (antenna1Short ? 0x10 : 0);
		packet.data[73] |= (byte) (antenna2Short ? 0x20 : 0);
		packet.data[73] |= (byte) (gnss1Failure ? 0x40 : 0);
		packet.data[73] |= (byte) (gnss2Failure ? 0x80 : 0);

		return packet;
	}
}
