/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Spatial SDK, Version 6.0                   */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

import java.util.Locale;

public class ANPacket3
{
	public static final int LENGTH = 24;

	public static final int DEVICE_ID_SPATIAL = 1;
	public static final int DEVICE_ID_ORIENTUS = 3;
	public static final int DEVICE_ID_SPATIAL_FOG = 4;
	public static final int DEVICE_ID_SPATIAL_DUAL = 5;
	public static final int DEVICE_ID_OBDII_ODOMETER = 10;
	public static final int DEVICE_ID_ORIENTUS_V3 = 11;
	public static final int DEVICE_ID_ILU = 12;
	public static final int DEVICE_ID_AIR_DATA_UNIT = 13;
	public static final int DEVICE_ID_SPATIAL_FOG_DUAL = 16;
	public static final int DEVICE_ID_MOTUS = 17;
	public static final int DEVICE_ID_GNSS_COMPASS = 19;
	public static final int DEVICE_ID_BLE_DONGLE = 24;
	public static final int DEVICE_ID_CERTUS = 26;
	public static final int DEVICE_ID_ARIES = 27;
	public static final int DEVICE_ID_BOREAS_D90 = 28;
	public static final int DEVICE_ID_BOREAS_D90_FPGA = 35;
	public static final int DEVICE_ID_BOREAS_COIL = 36;

	public long softwareVersion;
	public long deviceID;
	public long hardwareRevision;
	public long[] serialNumber;

	public ANPacket3()
	{
		softwareVersion = 0;
		deviceID = 0;
		hardwareRevision = 0;
		serialNumber = new long[3];
		serialNumber[0] = 0;
		serialNumber[1] = 0;
		serialNumber[2] = 0;
	}

	public ANPacket3(byte payload[])
	{
		softwareVersion = TypeConversion.bytesToUInt32(payload, 0);
		deviceID = TypeConversion.bytesToUInt32(payload, 4);
		hardwareRevision = TypeConversion.bytesToUInt32(payload, 8);
		serialNumber = new long[3];
		serialNumber[0] = TypeConversion.bytesToUInt32(payload, 12);
		serialNumber[1] = TypeConversion.bytesToUInt32(payload, 16);
		serialNumber[2] = TypeConversion.bytesToUInt32(payload, 20);
	}

	public String serialNumberString()
	{
		return String.format(Locale.ENGLISH, "%08X%08X%08X", serialNumber[0], serialNumber[1], serialNumber[2]);
	}

	public String deviceIdString()
	{
		switch ((int)deviceID)
		{
			case DEVICE_ID_SPATIAL:
				return "Spatial";
			case DEVICE_ID_ORIENTUS:
				return "Orientus";
			case DEVICE_ID_SPATIAL_FOG:
				return "Spatial FOG";
			case DEVICE_ID_SPATIAL_DUAL:
				return "Spatial Dual";
			case DEVICE_ID_OBDII_ODOMETER:
				return "OBDII Odometer";
			case DEVICE_ID_ORIENTUS_V3:
				return "Orientus";
			case DEVICE_ID_ILU:
				return "Interface & Logging Unit";
			case DEVICE_ID_AIR_DATA_UNIT:
				return "Air Data Unit";
			case DEVICE_ID_SPATIAL_FOG_DUAL:
				return "Spatial FOG Dual";
			case DEVICE_ID_MOTUS:
				return "Motus";
			case DEVICE_ID_GNSS_COMPASS:
				return "GNSS Compass";
			case DEVICE_ID_BLE_DONGLE:
				return "BLE Dongle";
			case DEVICE_ID_CERTUS:
				return "Certus";
			case DEVICE_ID_ARIES:
				return "Aries";
			case DEVICE_ID_BOREAS_D90:
				return "Boreas D90";
			case DEVICE_ID_BOREAS_D90_FPGA:
				return "Boreas D90 FPGA";
			case DEVICE_ID_BOREAS_COIL:
				return "Boreas Coil";
			default:
				return Long.toString(deviceID);
		}
	}

	public double hardwareRevisionDouble()
	{
		return hardwareRevision / 1000.0;
	}

	public double softwareVersionDouble()
	{
		return softwareVersion / 1000.0;
	}
}
