/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Spatial SDK, Version 6.0                   */
/*                    Copyright 2022, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2022 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* External Air Data */
public class ANPacket68
{
	public static final int LENGTH = 25;

	public float barometricAltitudeDelay;
	public float airspeedDelay;
	public float barometricAltitude;
	public float airspeed;
	public float barometricAltitudeStandardDeviation;
	public float airspeedStandardDeviation;
	public Boolean barometricAltitudeValid;
	public Boolean airspeedValid;
	public Boolean barometricAltitudeReferenceReset;

	public ANPacket68()
	{
	}

	public ANPacket68(ANPacket packet)
	{
		barometricAltitudeDelay = TypeConversion.bytesToFloat(packet.data, 0);
		airspeedDelay = TypeConversion.bytesToFloat(packet.data, 4);
		barometricAltitude = TypeConversion.bytesToFloat(packet.data, 8);
		airspeed = TypeConversion.bytesToFloat(packet.data, 12);
		barometricAltitudeStandardDeviation = TypeConversion.bytesToFloat(packet.data, 16);
		airspeedStandardDeviation = TypeConversion.bytesToFloat(packet.data, 20);
		barometricAltitudeValid = (packet.data[24] & 0x01) != 0;
		airspeedValid = (packet.data[24] & 0x02) != 0;
		barometricAltitudeReferenceReset = (packet.data[24] & 0x04) != 0;
	}
	
	public ANPacket toANPacket()
	{
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_AIR_DATA);
		TypeConversion.floatToBytes(packet.data, 0, barometricAltitudeDelay);
		TypeConversion.floatToBytes(packet.data, 4, airspeedDelay);
		TypeConversion.floatToBytes(packet.data, 8, barometricAltitude);
		TypeConversion.floatToBytes(packet.data, 12, airspeed);
		TypeConversion.floatToBytes(packet.data, 16, barometricAltitudeStandardDeviation);
		TypeConversion.floatToBytes(packet.data, 20, airspeedStandardDeviation);
		packet.data[25] = (byte) (barometricAltitudeValid ? 1 : 0);
		packet.data[25] |= (byte) (airspeedValid ? 1 : 0) << 1;
		packet.data[25] |= (byte) (barometricAltitudeReferenceReset ? 1 : 0) << 2;
		return packet;
	}
}
